@@ -1,68 +1,148 @@
 
+#include <glib/gi18n.h>
 #include <stdlib.h>
 #include <glade/glade.h>
-#include <libgnome/gnome-sound.h>
 #include <time.h>
 #include <string.h>
 
 #include "app.h"
 #include "e-phone-entry.h"
-
-#include <gnomebt-chooser.h>
+#include "gconf-bridge.h"
+#include "phonemgr-chooser-button.h"
+#include "phonemgr-utils.h"
 
 #define MAX_MESSAGE_LENGTH 160
 
-static gint conn_port=0;
-static gchar *bdaddr=NULL;
-
-static
-GladeXML *get_ui (MyApp *app, gchar *widget)
+static void
+boldify_label (GladeXML *xml, const char *name)
 {
-	gchar *fname;
-	GladeXML *ui;
+	GtkWidget *widget;
 
-	if (g_file_test ("../ui/phonemgr.glade", G_FILE_TEST_EXISTS))
-		fname = g_strdup ("../ui/phonemgr.glade");
-	else
-		fname = gnome_program_locate_file (app->program,
-					GNOME_FILE_DOMAIN_APP_DATADIR,
-					"phonemgr.glade", FALSE, NULL);
-	ui = glade_xml_new (fname, widget, NULL);
-	g_free (fname);
+	widget = glade_xml_get_widget (xml, name);
 
-	return ui;
+	if (widget == NULL) {
+		/* g_warning ("widget '%s' not found", name); */
+		return;
+	}
+
+	/* this way is probably better, but for some reason doesn't work with
+	 * labels with mnemonics.
+
+	static PangoAttrList *pattrlist = NULL;
+
+	if (pattrlist == NULL) {
+		PangoAttribute *attr;
+
+		pattrlist = pango_attr_list_new ();
+		attr = pango_attr_weight_new (PANGO_WEIGHT_BOLD);
+		attr->start_index = 0;
+		attr->end_index = G_MAXINT;
+		pango_attr_list_insert (pattrlist, attr);
+	}
+	gtk_label_set_attributes (GTK_LABEL (widget), pattrlist);*/
+
+	char *str_final;
+	str_final = g_strdup_printf ("<b>%s</b>", gtk_label_get_label (GTK_LABEL (widget)));
+	gtk_label_set_markup_with_mnemonic (GTK_LABEL (widget), str_final);
+	g_free (str_final);
 }
 
-static
-gchar *get_resource (MyApp *app, gchar *uiresname)
+static void
+bigger_label (GladeXML *xml, const char *name)
 {
-	gchar *fname;
+	GtkWidget *widget;
 
-	fname = gnome_program_locate_file (app->program,
-				GNOME_FILE_DOMAIN_APP_DATADIR,
-				uiresname,
-				TRUE, NULL);
+	widget = glade_xml_get_widget (xml, name);
 
-	if (fname == NULL)
-		fname = g_strdup_printf ("../ui/%s", uiresname);
+	if (widget == NULL) {
+		/* g_warning ("widget '%s' not found", name); */
+		return;
+	}
 
-	if (! g_file_test (fname, G_FILE_TEST_EXISTS)) {
-		fname = NULL;
+	char *str_final;
+	str_final = g_strdup_printf ("<span weight=\"bold\" size=\"larger\">%s</span>", gtk_label_get_label (GTK_LABEL (widget)));
+	gtk_label_set_markup_with_mnemonic (GTK_LABEL (widget), str_final);
+	g_free (str_final);
+}
+
+static char *
+get_resource (MyApp *app, char *uiresname)
+{
+	char *fname;
+
+	fname = g_build_filename ("..", "ui", uiresname, NULL);
+
+	if (g_file_test (fname, G_FILE_TEST_EXISTS) == FALSE) {
 		g_free (fname);
+		fname = g_build_filename (DATA_DIR, uiresname, NULL);
+	}
+
+	if (g_file_test (fname, G_FILE_TEST_EXISTS) == FALSE) {
+		g_free (fname);
+		fname = NULL;
 	}
 
 	return fname;
 }
 
+static
+GladeXML *get_ui (MyApp *app, char *widget)
+{
+	char *fname;
+	GladeXML *ui;
+
+	fname = get_resource (app, "phonemgr.glade");
+
+	ui = glade_xml_new (fname, widget, NULL);
+	g_free (fname);
+
+	boldify_label (ui, "alerting_label");
+	boldify_label (ui, "phone_connection_label");
+	boldify_label (ui, "error_handling_label");
+	bigger_label (ui, "new_message_label");
+	bigger_label (ui, "enter_message_label");
+
+	return ui;
+}
+
+static void
+play_sound (MyApp *app, const char *filename)
+{
+	GstElement *audio_sink;
+	char *uri;
+
+	/* Setup the playbin */
+	if (app->playbin == NULL) {
+		audio_sink = gst_element_factory_make ("gconfaudiosink", "audio-sink");
+		if (audio_sink == NULL)
+			audio_sink = gst_element_factory_make ("autoaudiosink", "audio-sink");
+		if (audio_sink == NULL) {
+			g_warning ("Couldn't create audio sink");
+			return;
+		}
+		if (g_object_class_find_property (G_OBJECT_GET_CLASS (audio_sink), "profile"))
+			g_object_set (G_OBJECT (audio_sink), "profile", 0, NULL);
+		app->playbin = gst_element_factory_make ("playbin", "play");
+		g_object_set (app->playbin, "audio-sink", audio_sink, NULL);
+	}
+
+	/* Play the file */
+	uri = g_filename_to_uri (filename, NULL, NULL);
+	gst_element_set_state (app->playbin, GST_STATE_NULL);
+	g_object_set (G_OBJECT (app->playbin), "uri", uri, NULL);
+	gst_element_set_state (app->playbin, GST_STATE_PLAYING);
+	g_free (uri);
+}
+
 static gboolean
 idle_play_alert (MyApp *app)
 {
-	gchar *fname;
+	char *fname;
 
 	if (gconf_client_get_bool (app->client, CONFBASE"/sound_alert", NULL)) {
-		fname = get_resource (app, "alert.wav");
+		fname = g_build_filename (DATA_DIR, "alert.wav", NULL);
 		if (fname) {
-			gnome_sound_play (fname);
+			play_sound (app, fname);
 			g_free (fname);
 		} else {
 			g_warning ("Couldn't find sound file %s", fname);
@@ -77,30 +157,17 @@ play_alert (MyApp *app)
 	g_idle_add ((GSourceFunc) idle_play_alert, (gpointer) app);
 }
 
-static gchar *
-get_current_btname (MyApp *app)
-{
-	gchar *name = NULL;
-	if (bdaddr) {
-		name = gnomebt_controller_get_device_preferred_name (
-				app->btctl, bdaddr);
-	}
-	if (!name)
-		name = g_strdup (_("No Bluetooth device chosen."));
-
-	return name;
-}
-
 static void
-set_dependent_widget (MyApp *app, gint conn_type, gboolean active)
+set_dependent_widget (MyApp *app, int conn_type, gboolean active)
 {
 	GtkWidget *w = NULL;
 	switch (conn_type) {
 		case CONNECTION_BLUETOOTH:
 			/* only set sensitive if bluetooth available */
-			if (app->btctl == NULL)
+			w = GTK_WIDGET (glade_xml_get_widget (app->ui, "btchooser"));
+			if (phonemgr_chooser_button_available (PHONEMGR_CHOOSER_BUTTON (w)) == FALSE
+			    || phonemgr_utils_connection_is_supported (PHONEMGR_CONNECTION_BLUETOOTH) == FALSE)
 				active = FALSE;
-			w = GTK_WIDGET (glade_xml_get_widget (app->ui, "btchoose"));
 			gtk_widget_set_sensitive (w, active);
 			break;
 		case CONNECTION_OTHER:
@@ -115,12 +182,16 @@ on_conn_port_change (GtkWidget *widget, MyApp *app)
 {
 	gboolean active = gtk_toggle_button_get_active (
 				GTK_TOGGLE_BUTTON (widget));
-	gint port = GPOINTER_TO_INT (
-			g_object_get_data (G_OBJECT (widget), "port"));
+	int port;
+	
+	port = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (widget), "port"));
 
 	set_dependent_widget (app, port, active);
-	if (active)
-		conn_port = port;
+	if (active) {
+		gconf_client_set_int (app->client,
+				      CONFBASE"/connection_type",
+				      port, NULL);
+	}
 }
 
 static gboolean
@@ -140,15 +211,6 @@ message_dialog_reply (GladeXML *ui)
 	return TRUE;
 }
 
-static void
-set_btdevname (MyApp *app)
-{
-	GtkWidget *w = GTK_WIDGET (glade_xml_get_widget (app->ui, "btdevname"));
-	gchar *c = get_current_btname (app);
-	gtk_label_set_text (GTK_LABEL (w), c);
-	g_free (c);
-}
-
 #define S_CONNECT(x, y) 	\
 	w = glade_xml_get_widget (app->ui, (x)); \
 	g_object_set_data (G_OBJECT (w), "port", GINT_TO_POINTER (y)); \
@@ -163,11 +225,11 @@ static void
 populate_prefs (MyApp *app)
 {
 	GtkWidget *w;
-	gchar *c;
-	gint ctype = gconf_client_get_int (app->client,
-			CONFBASE"/connection_type", NULL);
-
-	conn_port = ctype;
+	char *c, *bdaddr;
+	int ctype;
+	
+	ctype = gconf_client_get_int (app->client,
+				      CONFBASE"/connection_type", NULL);
 
 	set_dependent_widget (app, CONNECTION_BLUETOOTH,
 			ctype == CONNECTION_BLUETOOTH);
@@ -176,7 +238,7 @@ populate_prefs (MyApp *app)
 
 	w = GTK_WIDGET (glade_xml_get_widget (app->ui, "otherportentry"));
 	c  = gconf_client_get_string (app->client,
-				CONFBASE"/other_serial", NULL);
+				      CONFBASE"/other_serial", NULL);
 	if (c != NULL) {
 		gtk_entry_set_text (GTK_ENTRY (w), c);
 		g_free (c);
@@ -184,107 +246,34 @@ populate_prefs (MyApp *app)
 		gtk_entry_set_text (GTK_ENTRY (w), "");
 	}
 
-	w = GTK_WIDGET (glade_xml_get_widget (app->ui, "auto_retry"));
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w),
-			gconf_client_get_bool (app->client,
-				CONFBASE"/auto_retry", NULL));
-
-	w = GTK_WIDGET (glade_xml_get_widget (app->ui, "prefs_popup"));
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w),
-			gconf_client_get_bool (app->client,
-				CONFBASE"/popup_messages", NULL));
-
-	w = GTK_WIDGET (glade_xml_get_widget (app->ui, "prefs_sound"));
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w),
-			gconf_client_get_bool (app->client,
-				CONFBASE"/sound_alert", NULL));
-
-	/* TODO: if Bluetooth isn't available, disable the bluetooth radio */
-
 	S_ACTIVE("btdevice",  CONNECTION_BLUETOOTH);
 
-	w = GTK_WIDGET (glade_xml_get_widget (app->ui, "btdevice"));
-	if (app->btctl == NULL)
-		gtk_widget_set_sensitive (w, FALSE);
-
 	S_ACTIVE("serialport1", CONNECTION_SERIAL1);
 	S_ACTIVE("serialport2", CONNECTION_SERIAL2);
 	S_ACTIVE("irdaport", CONNECTION_IRCOMM);
 	S_ACTIVE("otherport", CONNECTION_OTHER);
-
-	if (bdaddr)
-		g_free (bdaddr);
-
-	bdaddr = gconf_client_get_string (app->client,
-			CONFBASE"/bluetooth_addr", NULL);
-	set_btdevname (app);
-
-
 }
 
 static void
-apply_prefs (MyApp *app, gpointer data)
+apply_prefs (MyApp *app)
 {
 	GtkWidget *w;
 
 	w = GTK_WIDGET (glade_xml_get_widget (app->ui, "otherportentry"));
 	gconf_client_set_string (app->client,
-			CONFBASE"/other_serial",
-			gtk_entry_get_text (GTK_ENTRY (w)), NULL);
-
-	w = GTK_WIDGET (glade_xml_get_widget (app->ui, "auto_retry"));
-	gconf_client_set_bool (app->client,
-			CONFBASE"/auto_retry",
-			gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w)), NULL);
-
-	gconf_client_set_int (app->client,
-			CONFBASE"/connection_type",
-			conn_port, NULL);
-
-	w = GTK_WIDGET (glade_xml_get_widget (app->ui, "prefs_popup"));
-	gconf_client_set_bool (app->client,
-			CONFBASE"/popup_messages",
-			gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w)), NULL);
-
-	w = GTK_WIDGET (glade_xml_get_widget (app->ui, "prefs_sound"));
-	gconf_client_set_bool (app->client,
-			CONFBASE"/sound_alert",
-			gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w)), NULL);
-
-	if (bdaddr != NULL)
-		gconf_client_set_string (app->client,
-				CONFBASE"/bluetooth_addr", bdaddr, NULL);
+				 CONFBASE"/other_serial",
+				 gtk_entry_get_text (GTK_ENTRY (w)), NULL);
 
 	reconnect_phone (app);
 }
 
 static void
-choose_bdaddr (MyApp *app, gpointer data)
+prefs_dialog_response (GtkWidget *dialog,
+		       int response,
+		       MyApp *app)
 {
-    gchar *ret = NULL;
-    GnomebtController *btctl;
-    GnomebtChooser *btchooser;
-    gint result;
-
-    btctl = app->btctl;
-    btchooser = gnomebt_chooser_new (btctl);
-    result = gtk_dialog_run (GTK_DIALOG (btchooser));
-
-    if (result == GTK_RESPONSE_OK)
-        ret = gnomebt_chooser_get_bdaddr (btchooser);
-
-    gtk_widget_destroy (GTK_WIDGET (btchooser));
-
-    /* consume events, so dialog gets removed from screen */
-    while (gtk_events_pending ())
-        gtk_main_iteration ();
-
-	if (ret) {
-		if (bdaddr)
-			g_free (bdaddr);
-		bdaddr = ret;
-		set_btdevname (app);
-	}
+	apply_prefs (app);
+	gtk_widget_hide (dialog);
 }
 
 static gboolean
@@ -303,9 +292,9 @@ set_send_sensitivity (GtkWidget *w, GladeXML *ui)
 	GtkTextBuffer *buf;
 	GtkWidget *sendbutton;
 	GtkLabel *left;
-	gint l;
-	gchar *work;
-	gchar *text;
+	int l;
+	char *work;
+	char *text;
 
 	left = GTK_LABEL (glade_xml_get_widget (ui, "charsleft"));
 	view = GTK_TEXT_VIEW (glade_xml_get_widget (ui, "messagebody"));
@@ -323,7 +312,7 @@ set_send_sensitivity (GtkWidget *w, GladeXML *ui)
 	if (l > MAX_MESSAGE_LENGTH) {
 		gtk_label_set_text (left, _("Message too long!"));
 	} else {
-		gint cl = MAX_MESSAGE_LENGTH - l;
+		int cl = MAX_MESSAGE_LENGTH - l;
 		work = g_strdup_printf ("%d", cl);
 		gtk_label_set_text (left, work);
 		g_free (work);
@@ -344,7 +333,7 @@ send_message (GtkWidget *w, GladeXML *ui)
 	EPhoneEntry *entry;
 	GtkDialog *dialog;
 	GtkTextIter s, e;
-	gchar *number;
+	char *number;
 
 	MyApp *app = (MyApp *) g_object_get_data (G_OBJECT (ui), "app");
 
@@ -365,7 +354,7 @@ send_message (GtkWidget *w, GladeXML *ui)
 }
 
 void
-create_send_dialog (MyApp *app, GtkDialog *parent, const gchar *recip)
+create_send_dialog (MyApp *app, GtkDialog *parent, const char *recip)
 {
 	GtkTextBuffer *buf;
 	GtkTextView *view;
@@ -375,14 +364,14 @@ create_send_dialog (MyApp *app, GtkDialog *parent, const gchar *recip)
 	GtkWidget *w;
 
 	ui = get_ui (app, "send_dialog");
-	
+
 	dialog = GTK_DIALOG (glade_xml_get_widget (ui, "send_dialog"));
 
 	if (parent) {
 		gtk_window_set_transient_for (GTK_WINDOW (dialog),
 			GTK_WINDOW (parent));
 	}
-	
+
 	view = GTK_TEXT_VIEW (glade_xml_get_widget (ui, "messagebody"));
 	buf = gtk_text_view_get_buffer (view);
 	gtk_text_buffer_set_text (buf, "", 0);
@@ -397,25 +386,24 @@ create_send_dialog (MyApp *app, GtkDialog *parent, const gchar *recip)
 	g_object_set_data (G_OBJECT (ui), "app", (gpointer) app);
 
 	/* hook up signals */
-	
 	g_signal_connect (G_OBJECT (dialog), "delete-event",
-			G_CALLBACK (gtk_widget_hide), (gpointer) dialog);
+			  G_CALLBACK (gtk_widget_hide), (gpointer) dialog);
 
 	g_signal_connect (G_OBJECT (dialog), "hide",
-			G_CALLBACK (destroy_send_dialog), (gpointer) ui);
+			  G_CALLBACK (destroy_send_dialog), (gpointer) ui);
 
 	w = GTK_WIDGET (glade_xml_get_widget (ui, "msgcancelbutton"));
 	g_signal_connect_swapped (G_OBJECT (w), "clicked",
-			G_CALLBACK (gtk_widget_hide), (gpointer) dialog);
+				  G_CALLBACK (gtk_widget_hide), (gpointer) dialog);
 
 	g_signal_connect (G_OBJECT (entry), "phone-changed",
-			G_CALLBACK (phone_number_changed), (gpointer) ui);
+			  G_CALLBACK (phone_number_changed), (gpointer) ui);
 	g_signal_connect (G_OBJECT (buf), "changed",
-			G_CALLBACK (set_send_sensitivity), (gpointer) ui);
+			  G_CALLBACK (set_send_sensitivity), (gpointer) ui);
 
 	w = GTK_WIDGET (glade_xml_get_widget (ui, "sendbutton"));
 	g_signal_connect (G_OBJECT (w), "clicked",
-			G_CALLBACK (send_message), (gpointer) ui);
+			  G_CALLBACK (send_message), (gpointer) ui);
 
 	gtk_widget_show_all (GTK_WIDGET (dialog));
 }
@@ -437,47 +425,53 @@ initialise_dequeuer (GConfClient *client, guint cnxn_id,
 void
 ui_init (MyApp *app)
 {
+	GConfBridge *bridge;
 	GtkWidget *w;
 
 	app->ui = get_ui (app, NULL);
+	bridge = gconf_bridge_get ();
 
 	if (!app->ui)
 		g_error ("Couldn't load user interface.");
 
 	/* close button and windowframe close button just hide the
 	   prefs panel */
-	
-	g_signal_connect_swapped (
-			G_OBJECT (glade_xml_get_widget (app->ui, "prefs_window")),
-			"delete-event", G_CALLBACK (gtk_widget_hide),
-			G_OBJECT (glade_xml_get_widget (app->ui, "prefs_window")));
+	g_signal_connect_swapped (G_OBJECT (glade_xml_get_widget (app->ui, "prefs_dialog")),
+				  "delete-event", G_CALLBACK (gtk_widget_hide),
+				  G_OBJECT (glade_xml_get_widget (app->ui, "prefs_dialog")));
 
-	g_signal_connect_swapped (
-			G_OBJECT (glade_xml_get_widget (app->ui, "prefsclosebutton")),
-			"clicked", G_CALLBACK (gtk_widget_hide),
-			G_OBJECT (glade_xml_get_widget (app->ui, "prefs_window")));
-
-	/* prefs apply */
-
-	g_signal_connect_swapped (
-			G_OBJECT (glade_xml_get_widget (app->ui, "prefsapplybutton")),
-			"clicked", G_CALLBACK (apply_prefs), (gpointer) app);
-
-	/* bt device chooser */
-
-	g_signal_connect_swapped (
-			G_OBJECT (glade_xml_get_widget (app->ui, "btchoose")),
-			"clicked", G_CALLBACK (choose_bdaddr), (gpointer) app);
+	/* response */
+	g_signal_connect (glade_xml_get_widget (app->ui, "prefs_dialog"),
+			  "response",
+     			  G_CALLBACK (prefs_dialog_response),
+			  app);
 
 	/* connect signal handlers for radio buttons */
-
 	S_CONNECT("btdevice",  CONNECTION_BLUETOOTH);
 	S_CONNECT("serialport1", CONNECTION_SERIAL1);
 	S_CONNECT("serialport2", CONNECTION_SERIAL2);
 	S_CONNECT("irdaport", CONNECTION_IRCOMM);
 	S_CONNECT("otherport", CONNECTION_OTHER);
 
-	app->tooltip = gtk_tooltips_new ();
+	/* Connect a few toggle buttons */
+	gconf_bridge_bind_property (bridge,
+				    CONFBASE"/auto_retry",
+				    G_OBJECT (glade_xml_get_widget (app->ui, "auto_retry")),
+				    "active");
+	gconf_bridge_bind_property (bridge,
+				    CONFBASE"/popup_messages",
+				    G_OBJECT (glade_xml_get_widget (app->ui, "prefs_popup")),
+				    "active");
+	gconf_bridge_bind_property (bridge,
+				    CONFBASE"/sound_alert",
+				    G_OBJECT (glade_xml_get_widget (app->ui, "prefs_sound")),
+				    "active");
+
+	/* And the address chooser */
+	gconf_bridge_bind_property (bridge,
+				    CONFBASE"/bluetooth_addr",
+				    G_OBJECT (glade_xml_get_widget (app->ui, "btchooser")),
+				    "device");
 
 	/* set up popup on message */
 	initialise_dequeuer (NULL, 0, NULL, app);
@@ -494,7 +488,7 @@ ui_hide (MyApp *app)
 	GtkWidget *w;
 	GdkDisplay *display;
 
-	w = glade_xml_get_widget (app->ui, "prefs_window");
+	w = glade_xml_get_widget (app->ui, "prefs_dialog");
 	display = gtk_widget_get_display (w);
 	gtk_widget_hide (w);
 
@@ -507,11 +501,19 @@ ui_hide (MyApp *app)
 void
 show_prefs_window (MyApp *app)
 {
-	GtkWidget *prefs = glade_xml_get_widget (app->ui, "prefs_window");
+	GtkWidget *prefs = glade_xml_get_widget (app->ui, "prefs_dialog");
 	populate_prefs (app);
 	gtk_widget_show (prefs);
 }
 
+static char *
+time_to_string (time_t time)
+{
+	char work[128];
+	strftime (work, sizeof(work), "%c", gmtime (&time));
+	return g_strdup (work);
+}
+
 gboolean
 dequeue_message (MyApp *app)
 {
@@ -521,8 +523,7 @@ dequeue_message (MyApp *app)
 	GtkTextView *textview;
 	GtkLabel *l_sender, *l_sent;
 	GtkTextBuffer *buf;
-	gchar work[64];
-	GDate *date;
+	char *time;
 	GladeXML *ui;
 	GtkWidget *w;
 
@@ -545,7 +546,7 @@ dequeue_message (MyApp *app)
 	w = GTK_WIDGET (glade_xml_get_widget (ui, "sms_dialog"));
 	g_signal_connect_swapped (G_OBJECT (w), "delete-event",
 			G_CALLBACK (gtk_widget_destroy), (gpointer) w);
-	
+
 	g_object_set_data (G_OBJECT (w), "app", (gpointer) app);
 
 	dialog = GTK_DIALOG (w);
@@ -566,11 +567,9 @@ dequeue_message (MyApp *app)
 	gtk_text_buffer_set_text (buf, msg->message, strlen (msg->message));
 	gtk_label_set_text (l_sender, msg->sender);
 
-	date = g_date_new ();
-	g_date_set_time (date, msg->timestamp);
-	g_date_strftime (work, 64, "%X %x", date);
-	g_date_free (date);
-	gtk_label_set_text (l_sent, work);
+	time = time_to_string (msg->timestamp);
+	gtk_label_set_text (l_sent, time);
+	g_free (time);
 
 	gtk_widget_show_all (GTK_WIDGET (dialog));
 
@@ -583,3 +582,4 @@ dequeue_message (MyApp *app)
 	
 	return TRUE;
 }
+
